# -*- coding: utf-8 -*-
import feedparser
import datetime
import logging
import redis
import json

from django.http import HttpResponse, Http404
from django.shortcuts import *
from django.contrib.auth.decorators import login_required
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.forms import UserCreationForm

from forms import MPConfigForm, ForbiddenForm
from moedjpack.moepad.mputils import rs
from moedjpack.moepad.mpdefs import *

log = logging.getLogger('moepad')


def register(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            form.save()  # new user created
            return HttpResponseRedirect("/accounts/login/")
    else:
        form = UserCreationForm()
    return render_to_response("registration/register.html",
                              {'form': form},
                              context_instance=RequestContext(request))


@login_required
def index(request):
    if request.method == 'POST':
        form = MPConfigForm(request.POST)
        if form.is_valid():
            rs.hmset("MoePadConf", request.POST)
        return render(request, "index.html", {"form": form})
    else:
        data = rs.hgetall("MoePadConf")
        form = MPConfigForm(initial=data)
        return render(request, "index.html", {"form": form})


@login_required
def forbidden_proc(request):
    print request.GET
    if request.method == 'POST':
        forbidden_keywords = request.POST['keywords'].split(',')
        f_set = set(forbidden_keywords)
        f_set.discard("")
        if f_set:
            for keyword in f_set:
                rs.sadd(FORBIDDENS, keyword)
    form = ForbiddenForm()
    forbidden_keywords = rs.smembers(FORBIDDENS)
    return render(request, "forbidden.html",
                  {"form": form, "forbiddens": forbidden_keywords})


@csrf_exempt
@login_required
def forbidden_item(request, item):
    if request.method == "DELETE":
        try:
            rs.srem(FORBIDDENS, item)
            status = 'ok'
        except:
            status = 'error'
    else:
        status = 'not implement'
    return HttpResponse(json.dumps({"status": status}),
                        content_type="application/json")


# def verify(request):
#     if request.method == 'POST':
#         print request.POST
#         for key in request.POST.keys():
#             if 'on' in request.POST[key]:
#                 RecentUpdateItems.objects.filter(title=key).update(
#                     itemState=RecentUpdateItems.VERIFIED)

#         return generateUnverifiedPage(request)
#     else:
#         return generateUnverifiedPage(request)


# def generateUnverifiedPage(request):
#     unverified_items_rec = RecentUpdateItems.objects.filter(
#         itemState=RecentUpdateItems.VERIFYING_NEW)
#     unverified_items = [rec.title.encode('utf-8')
#                         for rec in unverified_items_rec]
#     c = RequestContext(request, {"unverified_items": unverified_items})
#     return render_to_response('verify.html', c)



# -*- coding: utf-8 -*-
import os
from moedjpack.moepad.mputils import rs
from moedjpack.moepad.mpdefs import *


_dir = os.path.join(os.path.dirname(__file__), os.path.pardir)

logfile = os.path.join(_dir, "MoePad.log")


def showlog():
    os.system("tail -20 %s" % logfile)


def show_verifying():
    verifying_items = rs.zrange(VERIFYING_SET, 0, -1)
    for item in verifying_items:
        print(item)



from moedjpack.moepad.mpcelery import app
from mputils import logger
from update import UpdateItems, SendItem


@app.task
def send():
    sender = SendItem()
    sender.sendRoutine()


@app.task
def updateitem():
    update = UpdateItems(20)
    update.updateRoutine()


@app.task
def justtest():
    logger.info("justtest")


@app.task
def nighttest():
    logger.info("nighttest")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author: Sai
# @Date:   2014-02-13 22:16:44
# @Email:  email@example.com
# @Last modified by:   delo
# @Last modified time: 2014-03-01 15:09:23
from mputils import rs

MoePadConfKey = "MoePadConf"


class MpConfig(object):
    def __init__(self):
        super(MpConfig, self).__init__()

    def getConfig(self):
        # self.config = MoePadConfig.objects.all()[0]
        pass


class MpConfigRedisHandler(MpConfig):
    def __init__(self):
        super(MpConfigRedisHandler, self).__init__()
        self.getConfig()

    def getConfig(self):
        self.SinaAppKey = ""
        self.SinaAppSecret = ""
        self.TencentAppKey = ""
        self.TencentAppSecret = ""
        self.Domain = ""
        self.sameItemInterval = 24
        confdict = rs.hgetall(MoePadConfKey)
        for key in confdict:
            setattr(self, key, confdict[key])

MPConf = MpConfigRedisHandler()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__version__ = '1.1.2'
__author__ = 'Liao Xuefeng (askxuefeng@gmail.com)'

'''
Python client SDK for sina weibo API using OAuth 2.
'''

try:
    import json
except ImportError:
    import simplejson as json

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import gzip, time, hmac, base64, hashlib, urllib, urllib2, logging, mimetypes, collections

class APIError(StandardError):
    '''
    raise APIError if receiving json message indicating failure.
    '''
    def __init__(self, error_code, error, request):
        self.error_code = error_code
        self.error = error
        self.request = request
        StandardError.__init__(self, error)

    def __str__(self):
        return 'APIError: %s: %s, request: %s' % (self.error_code, self.error, self.request)

def _parse_json(s):
    ' parse str into JsonDict '

    def _obj_hook(pairs):
        ' convert json object to python object '
        o = JsonDict()
        for k, v in pairs.iteritems():
            o[str(k)] = v
        return o
    return json.loads(s, object_hook=_obj_hook)

class JsonDict(dict):
    ' general json object that allows attributes to be bound to and also behaves like a dict '

    def __getattr__(self, attr):
        try:
            return self[attr]
        except KeyError:
            raise AttributeError(r"'JsonDict' object has no attribute '%s'" % attr)

    def __setattr__(self, attr, value):
        self[attr] = value

def _encode_params(**kw):
    '''
    do url-encode parameters

    >>> _encode_params(a=1, b='R&D')
    'a=1&b=R%26D'
    >>> _encode_params(a=u'\u4e2d\u6587', b=['A', 'B', 123])
    'a=%E4%B8%AD%E6%96%87&b=A&b=B&b=123'
    '''
    args = []
    for k, v in kw.iteritems():
        if isinstance(v, basestring):
            qv = v.encode('utf-8') if isinstance(v, unicode) else v
            args.append('%s=%s' % (k, urllib.quote(qv)))
        elif isinstance(v, collections.Iterable):
            for i in v:
                qv = i.encode('utf-8') if isinstance(i, unicode) else str(i)
                args.append('%s=%s' % (k, urllib.quote(qv)))
        else:
            qv = str(v)
            args.append('%s=%s' % (k, urllib.quote(qv)))
    return '&'.join(args)

def _encode_multipart(**kw):
    ' build a multipart/form-data body with randomly generated boundary '
    boundary = '----------%s' % hex(int(time.time() * 1000))
    data = []
    for k, v in kw.iteritems():
        data.append('--%s' % boundary)
        if hasattr(v, 'read'):
            # file-like object:
            filename = getattr(v, 'name', '')
            content = v.read()
            data.append('Content-Disposition: form-data; name="%s"; filename="hidden"' % k)
            data.append('Content-Length: %d' % len(content))
            data.append('Content-Type: %s\r\n' % _guess_content_type(filename))
            data.append(content)
        else:
            data.append('Content-Disposition: form-data; name="%s"\r\n' % k)
            data.append(v.encode('utf-8') if isinstance(v, unicode) else v)
    data.append('--%s--\r\n' % boundary)
    return '\r\n'.join(data), boundary

def _guess_content_type(url):
    n = url.rfind('.')
    if n==(-1):
        return 'application/octet-stream'
    ext = url[n:]
    mimetypes.types_map.get(ext, 'application/octet-stream')

_HTTP_GET = 0
_HTTP_POST = 1
_HTTP_UPLOAD = 2

def _http_get(url, authorization=None, **kw):
    logging.info('GET %s' % url)
    return _http_call(url, _HTTP_GET, authorization, **kw)

def _http_post(url, authorization=None, **kw):
    logging.info('POST %s' % url)
    return _http_call(url, _HTTP_POST, authorization, **kw)

def _http_upload(url, authorization=None, **kw):
    logging.info('MULTIPART POST %s' % url)
    return _http_call(url, _HTTP_UPLOAD, authorization, **kw)

def _read_body(obj):
    using_gzip = obj.headers.get('Content-Encoding', '')=='gzip'
    body = obj.read()
    if using_gzip:
        logging.info('gzip content received.')
        gzipper = gzip.GzipFile(fileobj=StringIO(body))
        fcontent = gzipper.read()
        gzipper.close()
        return fcontent
    return body

def _http_call(the_url, method, authorization, **kw):
    '''
    send an http request and return a json object if no error occurred.
    '''
    params = None
    boundary = None
    if method==_HTTP_UPLOAD:
        # fix sina upload url:
        the_url = the_url.replace('https://api.', 'https://upload.api.')
        params, boundary = _encode_multipart(**kw)
    else:
        params = _encode_params(**kw)
        if '/remind/' in the_url:
            # fix sina remind api:
            the_url = the_url.replace('https://api.', 'https://rm.api.')
    http_url = '%s?%s' % (the_url, params) if method==_HTTP_GET else the_url
    http_body = None if method==_HTTP_GET else params
    req = urllib2.Request(http_url, data=http_body)
    req.add_header('Accept-Encoding', 'gzip')
    if authorization:
        req.add_header('Authorization', 'OAuth2 %s' % authorization)
    if boundary:
        req.add_header('Content-Type', 'multipart/form-data; boundary=%s' % boundary)
    try:
        resp = urllib2.urlopen(req)
        body = _read_body(resp)
        r = _parse_json(body)
        if hasattr(r, 'error_code'):
            raise APIError(r.error_code, r.get('error', ''), r.get('request', ''))
        return r
    except urllib2.HTTPError, e:
        try:
            r = _parse_json(_read_body(e))
        except:
            r = None
        if hasattr(r, 'error_code'):
            raise APIError(r.error_code, r.get('error', ''), r.get('request', ''))
        raise e

class HttpObject(object):

    def __init__(self, client, method):
        self.client = client
        self.method = method

    def __getattr__(self, attr):
        def wrap(**kw):
            if self.client.is_expires():
                raise APIError('21327', 'expired_token', attr)
            return _http_call('%s%s.json' % (self.client.api_url, attr.replace('__', '/')), self.method, self.client.access_token, **kw)
        return wrap

class APIClient(object):
    '''
    API client using synchronized invocation.
    '''
    def __init__(self, app_key, app_secret, redirect_uri=None, response_type='code', domain='api.weibo.com', version='2'):
        self.client_id = str(app_key)
        self.client_secret = str(app_secret)
        self.redirect_uri = redirect_uri
        self.response_type = response_type
        self.auth_url = 'https://%s/oauth2/' % domain
        self.api_url = 'https://%s/%s/' % (domain, version)
        self.access_token = None
        self.expires = 0.0
        self.get = HttpObject(self, _HTTP_GET)
        self.post = HttpObject(self, _HTTP_POST)
        self.upload = HttpObject(self, _HTTP_UPLOAD)

    def parse_signed_request(self, signed_request):
        '''
        parse signed request when using in-site app.

        Returns:
            dict object like { 'uid': 12345, 'access_token': 'ABC123XYZ', 'expires': unix-timestamp },
            or None if parse failed.
        '''

        def _b64_normalize(s):
            appendix = '=' * (4 - len(s) % 4)
            return s.replace('-', '+').replace('_', '/') + appendix

        sr = str(signed_request)
        logging.info('parse signed request: %s' % sr)
        enc_sig, enc_payload = sr.split('.', 1)
        sig = base64.b64decode(_b64_normalize(enc_sig))
        data = _parse_json(base64.b64decode(_b64_normalize(enc_payload)))
        if data['algorithm'] != u'HMAC-SHA256':
            return None
        expected_sig = hmac.new(self.client_secret, enc_payload, hashlib.sha256).digest();
        if expected_sig==sig:
            data.user_id = data.uid = data.get('user_id', None)
            data.access_token = data.get('oauth_token', None)
            expires = data.get('expires', None)
            if expires:
                data.expires = data.expires_in = time.time() + expires
            return data
        return None

    def set_access_token(self, access_token, expires):
        self.access_token = str(access_token)
        self.expires = float(expires)

    def get_authorize_url(self, redirect_uri=None, **kw):
        '''
        return the authorization url that the user should be redirected to.
        '''
        redirect = redirect_uri if redirect_uri else self.redirect_uri
        if not redirect:
            raise APIError('21305', 'Parameter absent: redirect_uri', 'OAuth2 request')
        response_type = kw.pop('response_type', 'code')
        return '%s%s?%s' % (self.auth_url, 'authorize', \
                _encode_params(client_id = self.client_id, \
                        response_type = response_type, \
                        redirect_uri = redirect, **kw))

    def request_access_token(self, code, redirect_uri=None):
        '''
        return access token as a JsonDict: {"access_token":"your-access-token","expires_in":12345678,"uid":1234}, expires_in is represented using standard unix-epoch-time
        '''
        redirect = redirect_uri if redirect_uri else self.redirect_uri
        if not redirect:
            raise APIError('21305', 'Parameter absent: redirect_uri', 'OAuth2 request')
        r = _http_post('%s%s' % (self.auth_url, 'access_token'), \
                client_id = self.client_id, \
                client_secret = self.client_secret, \
                redirect_uri = redirect, \
                code = code, grant_type = 'authorization_code')
        current = int(time.time())
        expires = r.expires_in + current
        remind_in = r.get('remind_in', None)
        if remind_in:
            rtime = int(remind_in) + current
            if rtime < expires:
                expires = rtime
        return JsonDict(access_token=r.access_token, expires=expires, expires_in=expires, uid=r.get('uid', None))

    def refresh_token(self, refresh_token):
        req_str = '%s%s' % (self.auth_url, 'access_token')
        r = _http_post(req_str, \
            client_id = self.client_id, \
            client_secret = self.client_secret, \
            refresh_token = refresh_token, \
            grant_type = 'refresh_token')
        current = int(time.time())
        expires = r.expires_in + current
        remind_in = r.get('remind_in', None)
        if remind_in:
            rtime = int(remind_in) + current
            if rtime < expires:
                expires = rtime
        return JsonDict(access_token=r.access_token, expires=expires, expires_in=expires, uid=r.get('uid', None))

    def is_expires(self):
        return not self.access_token or time.time() > self.expires

    def __getattr__(self, attr):
        if '__' in attr:
            return getattr(self.get, attr)
        return _Callable(self, attr)

_METHOD_MAP = { 'GET': _HTTP_GET, 'POST': _HTTP_POST, 'UPLOAD': _HTTP_UPLOAD }

class _Executable(object):

    def __init__(self, client, method, path):
        self._client = client
        self._method = method
        self._path = path

    def __call__(self, **kw):
        method = _METHOD_MAP[self._method]
        if method==_HTTP_POST and 'pic' in kw:
            method = _HTTP_UPLOAD
        return _http_call('%s%s.json' % (self._client.api_url, self._path), method, self._client.access_token, **kw)

    def __str__(self):
        return '_Executable (%s %s)' % (self._method, self._path)

    __repr__ = __str__

class _Callable(object):

    def __init__(self, client, name):
        self._client = client
        self._name = name

    def __getattr__(self, attr):
        if attr=='get':
            return _Executable(self._client, 'GET', self._name)
        if attr=='post':
            return _Executable(self._client, 'POST', self._name)
        name = '%s/%s' % (self._name, attr)
        return _Callable(self._client, name)

    def __str__(self):
        return '_Callable (%s)' % self._name

    __repr__ = __str__

if __name__=='__main__':
    import doctest
    doctest.testmod()



from django.contrib import admin
from moewiki.models import WikiItems, AlreadlyRetweeted, RecentUpdateItems, MoePadConfig

admin.site.register(WikiItems)
admin.site.register(AlreadlyRetweeted)
admin.site.register(RecentUpdateItems)
admin.site.register(MoePadConfig)


from django.db import models
import datetime
from django.utils import timezone

class WikiItems(models.Model):
    title = models.CharField(max_length=100)
    link = models.CharField(max_length=256)
    date = models.DateTimeField()

    def __unicode__(self):
        return self.title

    def was_add_recently(self):
        return self.date >= timezone.now() - datetime.timedelta(hours=24)


class AlreadlyRetweeted(models.Model):
    tid = models.CharField(max_length=32)
    source = models.CharField(max_length=20)
    date = models.DateTimeField()

    def __unicode__(self):
        return self.tid

    def was_add_recently(self):
        return self.date >= timezone.now() - datetime.timedelta(hours=2)


class RecentUpdateItems(models.Model):
    VERIFYING_NEW = 0
    VERIFIED = 1
    EDITED = 2
    SENT = 3
    ITEM_STATE = (
        (VERIFYING_NEW, 'VerifyingNewItem'),
        (VERIFIED, 'VerifiedItem'),
        (EDITED, 'Edited'),
        (SENT, 'Sent'),
    )

    title = models.CharField(max_length=128)
    itemState = models.IntegerField(default=EDITED, choices=ITEM_STATE)
    changeTime = models.DateTimeField()
    sentTime = models.DateTimeField(blank=True, null=True)

    def __unicode__(self):
        return self.title


class MoePadConfig(models.Model):

    SinaAppKey = models.CharField(max_length=32, blank=True)
    sinaAppSecret = models.CharField(max_length=64, blank=True)
    TencentAppKey = models.CharField(max_length=32, blank=True)
    TencentAppSecret = models.CharField(max_length=64, blank=True)
    MoeWebsite = models.CharField(max_length=128)
    feedurl = models.CharField(max_length=512)
    mail_from = models.CharField(max_length=64, blank=True)
    mail_to = models.CharField(max_length=64, blank=True)
    smtpserver = models.CharField(max_length=64, blank=True)
    mail_user = models.CharField(max_length=50, blank=True)
    mail_passwd = models.CharField(max_length=50, blank=True)
    mcport = models.IntegerField(default=11211)
    sameItemInterval = models.IntegerField(default=24)
    generalRetainTime = models.IntegerField(default=48)


# -*- coding: utf-8 -*-
import logging
import redis
import pickle
from django.http import HttpResponse
from django.shortcuts import *
from django.contrib.auth.decorators import login_required

from moedjpack.moepad.mpconf import MPConf
from moedjpack.moepad.weibowrapper import WeiboAuthInfoRedis
from moedjpack.moepad import weibo
from moedjpack.moepad.mputils import rs

log = logging.getLogger('moepad')

SinaAppKey = MPConf.SinaAppKey
SinaAppSecret = MPConf.SinaAppSecret
MoePadSite = MPConf.Domain
OriWeiboAuth = WeiboAuthInfoRedis("original")
RtWeiboAuth = WeiboAuthInfoRedis("retweet")


def authSina(code):
    client = weibo.APIClient(SinaAppKey, SinaAppSecret,
                             MoePadSite+"/sinacallback")
    r = client.request_access_token(code)
    client.set_access_token(r.access_token, r.expires_in)
    ruid = client.account.get_uid.get()
    #can get user type when callback from sina
    #so save the type in memcache, as long as
    #only on account get oauthed at a time, this works
    current_user_type = rs.get("current_user_type")
    print current_user_type
    if current_user_type == "original":
        WeiboAuthObj = OriWeiboAuth
    elif current_user_type == "retweet":
        WeiboAuthObj = RtWeiboAuth
    else:
        log.error("invalid callback user type autSina")
        raise Exception
    WeiboAuthObj.set(
        access_token=r.access_token,
        expires_in=r.expires_in,
        user_type=current_user_type, uid=ruid.uid)
    WeiboAuthObj.save()
    return r.access_token, r.expires_in


def authSinaUrl():
    client = weibo.APIClient(
        SinaAppKey, SinaAppSecret, MoePadSite+"/sinacallback")
    return client.get_authorize_url()


@login_required
def sinacallback(request):
    code = request.GET['code']
    access_token, expires_in = authSina(code)
    c = {"information": "授权信息已保存"}
    return render_to_response("info.html", c)


@login_required
def re_auth_sina(requset):
    WeiboAuthObj.clean()
    return redirect(authSinaUrl())


@login_required
def clean_auth(requset):
    rs.delete('WeiboAuthoriginal')
    return render_to_response("info.html",
                              {'information': '授权信息已清除'})


def getWeiboAuthedApi(source, user_type):
    if user_type == "original":
        weiboData = OriWeiboAuth
    elif user_type == "retweet":
        weiboData = RtWeiboAuth
    else:
        log.error("invalid user type getWeiboAuthedApi")
        raise e

    if source == 'sina':
        client = weibo.APIClient(
            SinaAppKey, SinaAppSecret, MoePadSite+"/sinacallback")
        client.set_access_token(weiboData.access_token, weiboData.expires_in)

        return client


@login_required
def loginSina(request, user_type):
    rs.set("current_user_type", user_type)
    return redirect(authSinaUrl())



# -*- coding: utf-8 -*-
import sys
import datetime
import requests
import json
import time
import calendar
import urllib
import urllib2
import logging
import traceback
from BeautifulSoup import BeautifulSoup

from weibowrapper import WeiboApi, WeiboAuthInfoRedis
from weibo import APIError
from mputils import rs, loggerInit, utcnow, _deletePrefix, logger
from mpconf import MPConf
from mpdefs import *


def getItemCategories(title):
    url = queryCategoryUrl % urllib.quote(title.encode('utf-8'))
    r = requests.get(url)
    rjson = json.loads(r.text)
    pages = rjson['query']['pages']
    key = pages.keys()[0]
    try:
        categories = pages[key]['categories']
    except KeyError:
        # category may not exist
        categories = []

    categories = [c['title'].partition('Category:')[2] for c in categories]
    return categories


def getImage(link):
    try:
        c = urllib2.urlopen(link)
    except:
        return None
    f = c.read()
    c.close()

    soup = BeautifulSoup(f)
    content = soup.find("div", {"id": "bodyContent"})
    img = None
    for tag in content.findAll('img'):
        try:
            if ((int(tag["width"]) < 100) or (int(tag["width"]) > 800)):
                continue
        except:
            continue
        try:
            if ((int(tag["height"]) < 100) or (int(tag["height"]) > 800)):
                continue
        except:
            continue
        img = tag
        break

    result = False
    if img:
        src = str(img["src"])
        if src.startswith("//1-ps.google"):
            src = img["pagespeed_lazy_src"]
        elif src.startswith("//"):
            src = "http:" + src

        image_remote = urllib2.urlopen(src)
        if image_remote.getcode() != 200:
            return None
        image_content = image_remote.read()
        with open("tmp.jpg", 'wb') as fpic:
            fpic.write(image_content)
        result = True
        # resize img ?
    return result


def filterForbiddenItems(that):
    forbiddenKeys = rs.smembers(FORBIDDENS)
    for key in forbiddenKeys:
        if key.decode('utf-8') in that:
            return False

    thatCategories = getItemCategories(that)
    if len(set(forbiddenKeys).intersection(set(thatCategories))):
        return False
    return True


def filterExistedItems(that):
    existedKeys = rs.keys(WIKIITEM_PREFIX+"*")
    existedItems = []
    if existedKeys:
        existedItems = rs.mget(existedKeys)
    verifyingItems = rs.zrange(VERIFYING_SET, 0, -1)
    existedItems += verifyingItems
    for item in existedItems:
        if that == item.decode('utf-8'):
            return False
    return True


def filterRedirectedItems(that):
    url = queryRedirectUrl % that.encode("utf-8")
    r = requests.get(url)
    rjson = json.loads(r.text)
    try:
        logger.info("redirected item: %s" % rjson['query']['redirects'][0]['from'].encode('utf-8'))
        return False
    except KeyError:
        return True


def verifyingKeyExpired(verifyingKey):
    expire_time = rs.zscore(VERIFYING_SET, verifyingKey)
    if not expire_time:
        logger.info("verifyingKey not in verifying keys set")
        return False

    if expire_time < time.time():
        return True
    return False


def autoVerifyExpiredItems():
    verifyingSet = rs.zrange(VERIFYING_SET, 0, -1)
    for verifyingKey in verifyingSet:
        if verifyingKeyExpired(verifyingKey):
            rs.zrem(VERIFYING_SET, verifyingKey)
            rs.set(NEWITEM+verifyingKey, verifyingKey, VERIFIED_EXPIRE)


def item_deleted(that):
    url = queryIfExistUrl % that.encode('utf-8')
    r = requests.get(url)
    rjson = json.loads(r.text)
    try:
        rjson['query']['pages']['-1']['missing']
        return True
    except KeyError:
        return False


def cleanDeletedItemsByPrefix(prefix):
    items = rs.keys(prefix+"*")
    for item in items:
        title = item.partition(prefix)[2]
        if item_deleted(title.decode('utf-8')):
            logger.info('check deleted: %s, %s' % (prefix, title))
            rs.delete(item)


def cleanDeletedItems():
    cleanDeletedItemsByPrefix(NEWITEM)
    cleanDeletedItemsByPrefix(EDITED)


def getItemTobeSend():
    cleanDeletedItems()
    new_items = rs.keys(NEWITEM+"*")
    if new_items:
        return NEWITEM, new_items[0]

    edited_items = rs.keys(EDITED+"*")
    if edited_items:
        return EDITED, edited_items[0]

    return None, None


def get_last_editor_name(item):
    url = queryLastRevisionEditor % item
    r = requests.get(url)
    rjson = json.loads(r.text)
    pages = rjson['query']['pages']
    page = pages[pages.keys()[0]]
    user_name = page['revisions'][0]['user']
    return user_name  # unicode here


def get_weibo_id_by_user_name(user_name):
    url = queryWeiboId % user_name
    r = requests.get(url)

    return "a"


class UpdateItems(object):

    filters = [
        filterForbiddenItems,
        filterExistedItems,
        filterRedirectedItems, ]

    def __init__(self, mins=20):
        super(UpdateItems, self).__init__()
        self.rch = []
        self.lastmins = mins

    def updateRoutine(self):
        try:
            self.fetchNewItemGenerated(self.lastmins)
            self.filter_valid()
            self.storeValidItems()
            autoVerifyExpiredItems()
            cleanDeletedItems()
        except:
            logger.info(traceback.format_exc())

    def fetchNewItemGenerated(self, mins):
        url_t = "http://zh.moegirl.org/api.php?format=json&action=query&list=recentchanges&rcstart=%s&rcend=%s&rcdir=newer&rcnamespace=0&rctoponly&rctype=edit|new"
        rcstart = calendar.timegm((utcnow() - datetime.timedelta(
            minutes=mins)).utctimetuple())
        rcend = calendar.timegm(utcnow().utctimetuple())
        url = url_t % (rcstart, rcend)
        print url
        r = requests.get(url)
        rjson = json.loads(r.text)
        self.rch = rjson['query']['recentchanges']
        self.new_items = [item['title'] for item in self.rch if item['type']
                          == 'new']
        self.edited_items = [item['title'] for item in self.rch if item['type']
                             != 'new']
        msg = '，'.join([item['title'].encode('utf-8') for item in self.rch])
        if msg:
            msg = ('过去%d分钟更新的条目有：' % mins)+msg
        else:
            msg = "过去%d分钟没有条目更新" % mins
        logger.info(msg)

    def filter_valid(self):
        for filterfunc in self.filters:
            self.new_items = filter(filterfunc, self.new_items)
            self.edited_items = filter(filterfunc, self.edited_items)

    def storeValidItems(self):
        for item in self.edited_items:
            itemKey = EDITED + item
            rs.set(EDITED+item, item, EDITED_EXPIRE)
            msg = "插入旧词条更新：%s" % item.encode('utf-8')
            logger.info(msg)

        for item in self.new_items:
            rs.zadd(VERIFYING_SET, item, time.time()+VERIFYING_ZSET_SCORE)
            msg = "插入新创建词条：%s" % item.encode('utf-8')
            logger.info(msg)


class SendItem(object):
    def __init__(self):
        super(SendItem, self).__init__()
        prefix, self.ItemTobeSendKey = getItemTobeSend()
        if not self.ItemTobeSendKey:
            setattr(self, 'ItemTobeSend', None)
            logger.info("No item to be send")
            return None
        self.ItemTobeSend = self.ItemTobeSendKey.partition(prefix)[2]
        self.getWeiboApi()

    def getWeiboApi(self):
        AuthInfo = WeiboAuthInfoRedis("original")
        token = rs.hgetall("WeiboAuth"+"original")
        self.weiboApi = WeiboApi(
            MPConf.SinaAppKey,
            MPConf.SinaAppSecret,
            MPConf.Domain+'/sinacallback',
            token['access_token'],
            token['expires_in'])

    def sendRoutine(self):
        if not self.ItemTobeSend:
            return None
        try:
            self.send()
            self.postsend()
            logger.info("Sending: "+self.ItemTobeSend+" Succ")
        except APIError as e:
            if 'expired_token' in e:
                logger.info("Token已到期，请重新授权")
        except:
            logger.info(traceback.format_exc())

    def send(self):
        weiboTitle = self.ItemTobeSend
        weiboLink = "http://zh.moegirl.org/" + \
                    urllib.quote(weiboTitle)
        last_editor = get_last_editor_name(self.ItemTobeSend)

        self.weiboApi.send(u'［' + weiboTitle.decode('utf-8') + u'］ 本条目最新一次编辑由' + last_editor + u'贡献',
                           weiboLink, getImage(weiboLink))

    def postsend(self):
        key = SENT + self.ItemTobeSend
        rs.set(key, self.ItemTobeSend, SENT_EXPIRE)
        rs.delete(self.ItemTobeSendKey)


def update():
    try:
        mins = int(sys.argv[1])
        updater = UpdateItems(mins)
    except:
        updater = UpdateItems()
    updater.updateRoutine()


def send():
    sender = SendItem()
    sender.sendRoutine()


def deletePrefix():
    for key in sys.argv[1:]:
        _deletePrefix(key)


if __name__ == '__main__':
    update()
    send()
